/*
 * this uses some ideas borrowed from IzPack
 */


/*
 *
 */
package installer.common;


/*
 *
 */
import java.awt.BorderLayout;
import java.awt.GraphicsEnvironment;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.EnumSet;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.WindowConstants;


/*
 * FIXME: it's a bit silly that the logging window is also owned here
 */
public class InstallerWindow
{
    /*
     * passed to panels to let them update whether the next/prev.
     * buttons can be clicked
     *
     * FIXME: make this a singleton
     */
    public class NavUpdateListener
    {
        //
        public void setNextButtonEnabled(boolean enabled)
        {
            nextPanelButton.setEnabled(enabled);
        }

        //
        public void setPreviousButtonEnabled(boolean enabled)
        {
            prevPanelButton.setEnabled(enabled);
        }

        //
        public void setQuitButtonQuiet(boolean quiet)
        {
            quitButtonQuiet = quiet;
        }

        //
        public void setQuitButtonToOK(boolean setToOK)
        {
            if (setToOK)
            {
                quitButton.setText("OK");
            }
            else
            {
                quitButton.setText("Quit");
            }
        }
    }


    /*
     * FIXME: i am not thrilled about making an array to ease getNext()
     *        and getPrev(), but i don't see a much better alternative
     */
    protected enum InstallerPanelID
    {
        //
        WELCOME_PANEL(new WelcomePanel()),
        SEARCH_FOR_MATLABS_PANEL(new SearchForMatlabsPanel()),
        PERFORM_INSTALLATION_PANEL(new PerformInstallationPanel());
        
        //
        private static final InstallerPanelID[] orderedPanels;

        //
        private final InstallerPanel panel;

        //
        static
        {
            EnumSet<InstallerPanelID> panelsSet = EnumSet.allOf(InstallerPanelID.class);

            orderedPanels = new InstallerPanelID[panelsSet.size()];
            panelsSet.toArray(orderedPanels);
        }

        //
        InstallerPanelID(InstallerPanel panel)
        {
            this.panel = panel;
        }

        //
        public InstallerPanelID getNext()
        {
            if (ordinal() + 1 >= orderedPanels.length)
            {
                return null;
            }
            else
            {
                return orderedPanels[ordinal() + 1];
            }
        }

        //
        public InstallerPanel getPanel()
        {
            return this.panel;
        }

        //
        public InstallerPanelID getPrevious()
        {
            if (ordinal() - 1 < 0)
            {
                return null;
            }
            else
            {
                return orderedPanels[ordinal() - 1];
            }
        }
    }


    /*
     *
     */
    private static final int GLOBAL_HEIGHT = 480;
    private static final int GLOBAL_WIDTH  = 640;

    /*
     *
     */
    private JFrame mainFrame;
    private JPanel panelContainer;

    /*
     *
     */
    private JButton nextPanelButton;
    private JButton prevPanelButton;
    private JButton quitButton;
    private JButton viewLogButton;

    /*
     *
     */
    private NavUpdateListener navUpdateListener;

    /*
     *
     */
    private InstallerPanelID currentPanelID;

    /*
     *
     */
    private LogWindow logWindow;
    private boolean logWindowIsVisible;

    /*
     *
     */
    private boolean quitButtonQuiet;
    
    /*
     *
     */
    private InstallerGUI gui;

    /*
     *
     */
    private Controller nativeController;


    /*
     *
     */
    public InstallerWindow(String title,
                           InstallerGUI installerGUI,
                           Controller nativeController,
                           Logger logger)
    {
        this.gui = installerGUI;
        this.nativeController = nativeController;

        for (InstallerPanelID panelID : EnumSet.allOf(InstallerPanelID.class))
        {
            panelID.getPanel().setGUI(this.gui);
        }

        this.mainFrame = new JFrame(title);

        this.logWindow = new LogWindow("WFDB Tools for MATLAB - Installer Log", logger);
        this.logWindowIsVisible = false;
        this.logWindow.hide();
        
        this.quitButtonQuiet = false;

        this.currentPanelID = InstallerPanelID.WELCOME_PANEL;
        buildMainGUI();
        this.navUpdateListener = new NavUpdateListener();
        switchToPanel(this.currentPanelID);
    }


    /*
     *
     */
    private void buildMainGUI()
    {
        JPanel navPanel;

        
        //
        this.mainFrame.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);

        //
        try
        {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        }
        catch (ClassNotFoundException ex) {}
        catch (InstantiationException ex) {}
        catch (IllegalAccessException ex) {}
        catch (UnsupportedLookAndFeelException ex) {}

        //
        this.mainFrame.getContentPane().setLayout(new BorderLayout());

        //
        this.panelContainer = new JPanel();
        this.panelContainer.setBorder(BorderFactory.createEmptyBorder(10, 10, 0, 10));
        this.panelContainer.setLayout(new GridLayout(1, 1));
        this.mainFrame.getContentPane().add(panelContainer, BorderLayout.CENTER);

        //
        navPanel = new JPanel();
        navPanel.setLayout(new BoxLayout(navPanel, BoxLayout.X_AXIS));
        navPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 0, 10));
        this.mainFrame.getContentPane().add(navPanel, BorderLayout.SOUTH);

        //
        this.viewLogButton = new JButton("View Log");
        this.viewLogButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    toggleLogWindow();
                }
            });

        this.prevPanelButton = new JButton("Previous");
        this.prevPanelButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    moveToPreviousPanel();
                }
            });
        
        this.nextPanelButton = new JButton("Next");
        this.nextPanelButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    moveToNextPanel();
                }
            });

        this.quitButton = new JButton("Quit");
        this.quitButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    handleQuitRequest();
                }
            });

        //
        navPanel.add(this.viewLogButton);
        navPanel.add(Box.createHorizontalGlue());
        navPanel.add(this.prevPanelButton);
        navPanel.add(this.nextPanelButton);
        // FIXME: insert a bit of space here
        navPanel.add(this.quitButton);
        this.prevPanelButton.setEnabled(false);
        this.nextPanelButton.setEnabled(false);
    }


    /*
     *
     */
    private void handleQuitRequest()
    {
        boolean performQuit;
        int returnValue;


        //
        if (!quitButtonQuiet)
        {
            int result =
                JOptionPane.showConfirmDialog(this.mainFrame,
                                              "The installation is not completed;\n "
                                              + "you may also be interrupting any "
                                              + "pending operations.\n "
                                              + "Are you sure you want to quit?",
                                              "Cancel Installation?",
                                              JOptionPane.YES_NO_OPTION);
            performQuit = (result == JOptionPane.YES_OPTION);
            returnValue = 1;
        }
        else
        {
            File wsmCodeDir;
            File wsmCodeUtilDir;
            String wsmCodeDirPath;
            String wsmCodeUtilDirPath;

            performQuit = true;
            returnValue = 0;

            wsmCodeDir = nativeController.getWSMCodeDirectory();
            wsmCodeUtilDir = nativeController.getWSMCodeUtilDirectory();
            wsmCodeDirPath = wsmCodeDir.getAbsolutePath();
            wsmCodeUtilDirPath = wsmCodeUtilDir.getAbsolutePath();

            // tell the user about matlab paths that were unable to be set up
            for (Map.Entry<File, InstallerGUI.MatlabInstallation> matlabInstall :
                     gui.getMatlabInstallations().entrySet())
            {
                if ((matlabInstall.getValue().lastFinishedStep ==
                     InstallerGUI.InstallationStep.TESTING)
                    && (!matlabInstall.getValue().pathUpdated))
                {
                    JOptionPane.showMessageDialog(this.mainFrame,
                                                  "The installer may not have been able to "
                                                  + "update the path properly for\n"
                                                  + "MATLAB version "
                                                  + matlabInstall.getValue().fullVersion + " "
                                                  + "(in directory\n"
                                                  + matlabInstall.getKey().getAbsolutePath()
                                                  + ") -- you may need to update it yourself.\n"
                                                  + "You can do so by adding the two directories,\n"
                                                  + wsmCodeDirPath + " and " + wsmCodeUtilDirPath
                                                  + "\n"
                                                  + "to the path in MATLAB; you can use the command "
                                                  + "\"pathtool\" to help do this.\n"
                                                  + "Sorry for the inconvenience!\n",
                                                  "Path Not Updated for MATLAB",
                                                  JOptionPane.INFORMATION_MESSAGE);
                }
            }

            // ask to run final demo
            if (someMatlabCompletedSuccessfully())
            {
                File matlabPath;

                matlabPath = showFinalMatlabLaunchDialog();
                if (matlabPath != null)
                {
                    try
                    {
                        nativeController.launchMatlabWithCommand(matlabPath,
                                                                 "demo_wfdb_tools",
                                                                 wsmCodeDir);
                    }
                    catch (IOException ex)
                    {
                        JOptionPane.showMessageDialog(this.mainFrame,
                                                      "An error occurred while launching Matlab: \n"
                                                      + ex.getMessage() + "\n\n"
                                                      + "You may try running it on your own.",
                                                      "Error Launching Matlab",
                                                      JOptionPane.ERROR_MESSAGE);
                        Logger.getLogger("controller").severe("launching Matlab failed: "
                                                              + ex.getMessage() + "\n");
                    }
                }
            }
        }

        //
        if (performQuit)
        {            
            this.mainFrame.setVisible(false);
            this.logWindow.hide();
            this.mainFrame.dispose();
            this.logWindow.dispose();
            System.exit(returnValue);
        }
    }


    /*
     *
     */
    private void moveToPreviousPanel()
    {
        InstallerPanelID prevPanelID;

        
        prevPanelID = this.currentPanelID.getPrevious();
        if (prevPanelID != null)
        {
            switchToPanel(prevPanelID);
        }
    }


    /*
     *
     */
    private void moveToNextPanel()
    {
        InstallerPanelID nextPanelID;

        
        nextPanelID = this.currentPanelID.getNext();
        if (nextPanelID != null)
        {
            switchToPanel(nextPanelID);
        }
    }


    /*
     *
     */
    public void show()
    {
        //
        GraphicsEnvironment graphicsEnvironment;
        Point geCenter;


        this.mainFrame.setSize(GLOBAL_WIDTH, GLOBAL_HEIGHT);
        this.mainFrame.setResizable(false);

        graphicsEnvironment = GraphicsEnvironment.getLocalGraphicsEnvironment();
        geCenter = graphicsEnvironment.getCenterPoint();
        this.mainFrame.setBounds(Math.max(geCenter.x - this.mainFrame.getWidth() / 2, 0),
                                 Math.max(geCenter.y - this.mainFrame.getHeight() / 2, 0),
                                 this.mainFrame.getWidth(),
                                 this.mainFrame.getHeight());

        this.mainFrame.setVisible(true);
    }


    /*
     * return the path of Matlab with which to launch the final test script,
     * or null if it should not be launched
     */
    private File showFinalMatlabLaunchDialog()
    {
        Map<JRadioButton, File> buttons = new HashMap<JRadioButton, File>();
        Box buttonsBox = Box.createVerticalBox();
        ButtonGroup buttonsGroup = new ButtonGroup();
        boolean first = true;
        int result;


        //
        for (Map.Entry<File, InstallerGUI.MatlabInstallation> matlabInstall :
                 gui.getMatlabInstallations().entrySet())
        {
            if ((matlabInstall.getValue().lastFinishedStep ==
                 InstallerGUI.InstallationStep.TESTING)
                && (!matlabInstall.getValue().hasFailed))
            {
                JRadioButton button = new JRadioButton(matlabInstall.getValue().fullVersion);
                // FIXME: need to know if this is the only install of this version;
                //        if not, should display the path, too, e.g. "(in /opt/matlab/ ...)"
                buttons.put(button, matlabInstall.getKey());
                buttonsBox.add(button);
                buttonsGroup.add(button);
                if (first)
                {
                    button.setSelected(true);
                    first = false;
                }
            }
        }

        //
        assert (buttons.size() != 0);

        //
        result =
            JOptionPane.showConfirmDialog(this.mainFrame,
                                          new Object[] {
                                              "Would you like to launch MATLAB now,\n"
                                              + " and run a short test of WFDB Tools?",
                                              buttonsBox
                                          },
                                          "Run WFDB Test in MATLAB?",
                                          JOptionPane.YES_NO_OPTION);
        if (result == JOptionPane.NO_OPTION)
        {
            return null;
        }

        //
        for (Map.Entry<JRadioButton, File> buttonAndFile : buttons.entrySet())
        {
            if (buttonAndFile.getKey().isSelected())
            {
                return buttonAndFile.getValue();
            }
        }

        // if we get here, we've run into a bug
        // (a java bug, i would think)
        assert (false);

        //
        return null;
    }


    /*
     * return true if wfdb-swig-matlab was successfully
     * installed and tested for some version of matlab
     */
    private boolean someMatlabCompletedSuccessfully()
    {
        for (Map.Entry<File, InstallerGUI.MatlabInstallation> matlabInstall :
                 gui.getMatlabInstallations().entrySet())
        {
            if ((matlabInstall.getValue().lastFinishedStep ==
                 InstallerGUI.InstallationStep.TESTING)
                && (!matlabInstall.getValue().hasFailed))
            {
                return true;
            }
        }
        
        return false;
    }


    /*
     * the word "switch" is slightly misleading, only
     * because this may be the first time we've installed
     * a panel
     */
    private void switchToPanel(InstallerPanelID panelID)
    {
        InstallerPanel nextPanel;


        this.prevPanelButton.setEnabled(false);
        this.nextPanelButton.setEnabled(true);

        this.navUpdateListener.setQuitButtonQuiet(false);
        this.navUpdateListener.setQuitButtonToOK(false);


        if (this.currentPanelID.getPanel() != null)
        {
            if (!this.currentPanelID.getPanel().unload())
            {
                return;
            }
            else
            {
                this.panelContainer.setVisible(false);
                this.panelContainer.remove(this.currentPanelID.getPanel().getJPanel());
            }
        }

        this.currentPanelID = panelID;
        this.currentPanelID.getPanel().preload();
        this.panelContainer.add(this.currentPanelID.getPanel().getJPanel());
        // this.currentPanelID.getPanel().getJPanel().setUI(this.panelContainer.getUI());
        this.panelContainer.validate();
        this.panelContainer.setVisible(true);
        this.currentPanelID.getPanel().postload(this.nativeController, this.navUpdateListener);
    }


    /*
     *
     */
    private void toggleLogWindow()
    {
        if (this.logWindowIsVisible)
        {
            this.logWindowIsVisible = false;
            this.viewLogButton.setText("View Log");
            this.logWindow.hide();
         }
        else
        {
            this.logWindowIsVisible = true;
            this.viewLogButton.setText("Hide Log");
            this.logWindow.show();
         }
    }
}