/*
 *
 */


/*
 *
 */
package installer.common;


/*
 *
 */
import java.awt.BorderLayout;
import java.awt.EventQueue;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.logging.Logger;
import javax.swing.Box;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import installer.common.GUIWorker;


/*
 * FIXMEFIXME: there will be a problem if there are multiple installations with the same
 *             version!
 */
public class PerformInstallationPanel extends InstallerPanel
{
    /*
     *
     */
    private boolean hasBeenLoadedBefore;


    /*
     * indicates whether this panel has successfully installed
     * the files for at least one installation of matlab
     */
    private boolean installedForSomeMatlab;


    /*
     *
     */
    private boolean tempDirMovedToFinalDir;


    // /*
    //  *
    //  */
    // private boolean userpathVariableUpdated;


    /*
     *
     */
    private boolean wfdbFilesCopied;


    /*
     *
     */
    private JLabel titleLabel;
    private Box mainVertBox;
    private VertScrollWidget mainScrollPane;
    private boolean summaryTextDone;


    /*
     *
     */
    public PerformInstallationPanel()
    {
        this.hasBeenLoadedBefore = false;
        this.installedForSomeMatlab = false;
        this.tempDirMovedToFinalDir = false;
        // this.userpathVariableUpdated = false;
        this.wfdbFilesCopied = false;
        this.summaryTextDone = false;

        buildPanelGUI();
    }


    /*
     *
     */
    private void buildPanelGUI()
    {
        //
        this.panel.setLayout(new BorderLayout());
        
        //
        this.titleLabel = new JLabel("<html><h2>Performing Installation</h2></html>");
        this.titleLabel.setHorizontalAlignment(JLabel.CENTER);
        this.panel.add(titleLabel, BorderLayout.NORTH);

        //
        this.mainVertBox = Box.createVerticalBox();
        this.mainScrollPane = new VertScrollWidget(this.mainVertBox);
        this.panel.add(this.mainScrollPane, BorderLayout.CENTER);
    }


    /*
     *
     */
    private String buildSummaryText()
    {
        StringBuffer buffer;
        boolean someMatlabFailedTest;
        boolean installedForSomeMatlab;


        //
        buffer = new StringBuffer();
        buffer.append("<html><font size=\"4\"><b><ul>");
        someMatlabFailedTest = false;
        installedForSomeMatlab = false;
        
        //
        for (Map.Entry<File, InstallerGUI.MatlabInstallation> matlabInstall :
                 gui.getMatlabInstallations().entrySet())
        {
            if ((matlabInstall.getValue().lastFinishedStep !=
                 InstallerGUI.InstallationStep.TESTING)
                || matlabInstall.getValue().hasFailed)
            {
                continue;
            }

            buffer.append("<li> MATLAB installation in " + matlabInstall.getKey().getAbsolutePath()
                          + ", which has version " + matlabInstall.getValue().fullVersion
                          + ", was set up and tested successfully.\n"
                          + "<br></br><br></br>\n");

            installedForSomeMatlab = true;
        }

        //
        for (Map.Entry<File, InstallerGUI.MatlabInstallation> matlabInstall :
                 gui.getMatlabInstallations().entrySet())
        {
            if ((matlabInstall.getValue().lastFinishedStep !=
                 InstallerGUI.InstallationStep.INSTALLATION)
                || (!matlabInstall.getValue().hasFailed))
            {
                continue;
            }

            buffer.append("<li> MATLAB installation in " + matlabInstall.getKey().getAbsolutePath()
                          + ", which has version " + matlabInstall.getValue().fullVersion
                          + ", was set up successfully, but failed the test script.\n"
                          + "<br></br><br></br>\n");

            someMatlabFailedTest = true;
            installedForSomeMatlab = true;
        }

        // FIXME: test this!
        for (Map.Entry<File, InstallerGUI.MatlabInstallation> matlabInstall :
                 gui.getMatlabInstallations().entrySet())
        {
            if ((matlabInstall.getValue().lastFinishedStep !=
                 InstallerGUI.InstallationStep.VERIFICATION)
                || (!matlabInstall.getValue().hasFailed))
            {
                continue;
            }

            buffer.append("<li> MATLAB installation in " + matlabInstall.getKey().getAbsolutePath()
                          + ", which has version " + matlabInstall.getValue().fullVersion
                          + ", was not able to be set up properly. Please check the log for details.\n"
                          + "<br></br><br></br>\n");
        }

        //
        if (someMatlabFailedTest)
        {
            buffer.append("<li> Please note that, if the test script failed for some MATLAB installation,\n"
                          + "   it does not necessarily indicate that the WFDB Tools were not set up\n"
                          + "   properly (for example, an intermittent network problem may have caused\n"
                          + "   the test script to fail.) You are encouraged to try WFDB Tools in that\n"
                          + "   installation of MATLAB.\n"
                          + "<br></br><br></br>\n");
        }

        // FIXME: test this
        if (!installedForSomeMatlab)
        {
            buffer.append("<li> WFDB Tools was not able to be installed for any version "
                          + "of MATLAB selected.\n"
                          + "<br></br><br></br>\n");
        }

        //
        buffer.append("</ul></b></font></html>\n");

        //
        return buffer.toString();
    }


    /*
     * get any MATLAB path for which WFDB was installed;
     * prefer paths that have been tested, too
     */
    private File getAnyInstalledMatlabPath()
    {
        for (Map.Entry<File, InstallerGUI.MatlabInstallation> matlabInstall :
                 gui.getMatlabInstallations().entrySet())
        {
            if (matlabInstall.getValue().lastFinishedStep ==
                InstallerGUI.InstallationStep.TESTING)
            {
                return matlabInstall.getKey();
            }
        }

        for (Map.Entry<File, InstallerGUI.MatlabInstallation> matlabInstall :
                 gui.getMatlabInstallations().entrySet())
        {
            if (matlabInstall.getValue().lastFinishedStep ==
                InstallerGUI.InstallationStep.INSTALLATION)
            {
                return matlabInstall.getKey();
            }
        }

        return null;
    }


    /*
     *
     */
    public void postload(final Controller nativeController,
                         final InstallerWindow.NavUpdateListener navUpdateListener)
    {
        final PerformInstallationPanel panelThis = this;
        final Logger logger = Logger.getLogger("controller");


        //
        navUpdateListener.setPreviousButtonEnabled(false);
        navUpdateListener.setNextButtonEnabled(false);
        this.panel.validate();

        //
        (new GUIWorker() {
                //
                private boolean installedForSomeMatlab = false;
                private boolean finishedForSomeMatlab = false;
                private boolean justFailedForSomeMatlab = false;
                private boolean failureCopyingNewArchs = false;

                //
                protected void doInBackground()
                {
                    try
                    {
                        Set<String> archs;

                        // get the set of all jvm archs. for which to install
                        archs = new HashSet<String>();
                        for (Map.Entry<File, InstallerGUI.MatlabInstallation> matlabInstall :
                                 gui.getMatlabInstallations().entrySet())
                        {
                            InstallerGUI.MatlabInstallation installInfo =
                                matlabInstall.getValue();
                            
                            if ((installInfo.lastFinishedStep ==
                                 InstallerGUI.InstallationStep.VERIFICATION)
                                && !installInfo.hasFailed)
                            {
                                archs.add(installInfo.jvmArch);
                            }
                        }
                        

                        //
                        //
                        //
                        if (panelThis.wfdbFilesCopied)
                        {
                            // some new archs may have been requested since
                            // the last time this panel was loaded
                            if (!archs.isEmpty())
                            {
                                workerPushLabel("Copying WFDB Files...");
                                
                                try
                                {
                                    nativeController.copyNativeLibsFromPackageToHost(archs);
                                }
                                catch (IOException ex)
                                {
                                    logger.severe("Problem copying files to host: "
                                                  + ex.getMessage() + "\n");
                                    failureCopyingNewArchs = true;
                                    return;
                                }
                                catch (ProcessExecutionException ex)
                                {
                                    logger.severe("Problem copying files to host: "
                                                  + ex.getMessage() + "\n");
                                    failureCopyingNewArchs = true;
                                    return;
                                }
                                catch (ResourceNotFoundException ex)
                                {
                                    logger.severe("Problem copying files to host: "
                                                  + ex.getMessage() + "\n");
                                    failureCopyingNewArchs = true;
                                    return;
                                }

                                workerPushLabel("WFDB files copied.");
                            }
                        }
                        else
                        {
                            //
                            workerPushLabel("Copying WFDB Files...");

                            //
                            try
                            {
                                nativeController.copyFilesFromPackageToHost();
                                nativeController.copyNativeLibsFromPackageToHost(archs);
                                panelThis.wfdbFilesCopied = true;
                                workerPushLabel("WFDB Files copied.");
                            }
                            catch (IOException ex)
                            {
                                logger.severe("Problem copying files to host: "
                                              + ex.getMessage() + "\n");
                                panelThis.wfdbFilesCopied = false;
                                workerPushLabel("WFDB Files not copied.");
                                return;
                            }
                            catch (ProcessExecutionException ex)
                            {
                                logger.severe("Problem copying files to host: "
                                              + ex.getMessage() + "\n");
                                panelThis.wfdbFilesCopied = false;
                                workerPushLabel("WFDB Files not copied.");
                                return;
                            }
                            catch (ResourceNotFoundException ex)
                            {
                                logger.severe("Problem copying files to host: "
                                              + ex.getMessage() + "\n");
                                panelThis.wfdbFilesCopied = false;
                                workerPushLabel("WFDB Files not copied.");
                                return;
                            }
                            catch (Exception ex)
                            {
                                // this should never happen; if it does, there's a bug.
                                // FIXME: abstract all of this
                                logger.severe("Unknown problem copying files to host (this is a bug!): "
                                              + ex.toString() + "\n");
                                logger.severe("Stack trace:\n");
                                for (StackTraceElement traceLine : ex.getStackTrace())
                                {
                                    logger.severe(traceLine.toString() + "\n");
                                }
                                panelThis.wfdbFilesCopied = false;
                                workerPushLabel("WFDB Files not copied.");
                                return;
                            }
                        }

                        //
                        // has wfdb already been installed for some matlab?
                        //
                        for (Map.Entry<File, InstallerGUI.MatlabInstallation> matlabInstall :
                                 gui.getMatlabInstallations().entrySet())
                        {
                            File path = matlabInstall.getKey();
                            InstallerGUI.MatlabInstallation installInfo = matlabInstall.getValue();
                            
                            if (((installInfo.lastFinishedStep !=
                                  InstallerGUI.InstallationStep.INSTALLATION)
                                 && (installInfo.lastFinishedStep !=
                                     InstallerGUI.InstallationStep.TESTING)))
                            {
                                continue;
                            }
                            
                            installedForSomeMatlab = true;
                            break;
                        }

                        //
                        // try to install wfdb for the matlabs that have been verified
                        // and selected
                        //
                        for (Map.Entry<File, InstallerGUI.MatlabInstallation> matlabInstall :
                                 gui.getMatlabInstallations().entrySet())
                        {
                            File path = matlabInstall.getKey();
                            InstallerGUI.MatlabInstallation installInfo = matlabInstall.getValue();
                            
                            if ((installInfo.lastFinishedStep ==
                                 InstallerGUI.InstallationStep.VERIFICATION)
                                && (!installInfo.hasFailed))
                            {
                                //
                                workerPushLabel("Installing for MATLAB version "
                                                + installInfo.fullVersion + " ...");

                                //
                                try
                                {
                                    nativeController.updateMatlabPathFiles(path,
                                                                           installInfo.fullVersion,
                                                                           installInfo.jvmArch);
                                }
                                catch (IOException ex)
                                {
                                    justFailedForSomeMatlab = true;
                                    installInfo.hasFailed = true;
                                    logger.warning("An error occurred: " + ex.getMessage() + "\n");
                                    workerPushLabel("Unable to install for MATLAB version "
                                                    + installInfo.fullVersion);
                                    continue;
                                }
                                catch (ProcessExecutionException ex)
                                {
                                    justFailedForSomeMatlab = true;
                                    installInfo.hasFailed = true;
                                    logger.warning("An error occurred: " + ex.getMessage() + "\n");
                                    workerPushLabel("Unable to install for MATLAB version "
                                                    + installInfo.fullVersion);
                                    continue;
                                }
                                catch (NoSuperuserPasswordException ex)
                                {
                                    justFailedForSomeMatlab = true;
                                    installInfo.hasFailed = true;
                                    logger.warning("An error occurred: " + ex.getMessage() + "\n");
                                    workerPushLabel("Unable to install for MATLAB version "
                                                    + installInfo.fullVersion);
                                    continue;
                                }

                                //
                                try
                                {
                                    nativeController.writeConfigFile(path, installInfo.fullVersion);
                                }
                                catch (IOException ex)
                                {
                                    justFailedForSomeMatlab = true;
                                    installInfo.hasFailed = true;
                                    logger.warning("An error occurred: " + ex.getMessage() + "\n");
                                    workerPushLabel("Unable to install for MATLAB version "
                                                    + installInfo.fullVersion);
                                    continue;
                                }

                                //
                                installedForSomeMatlab = true;
                                installInfo.lastFinishedStep =
                                    InstallerGUI.InstallationStep.INSTALLATION;
                                workerPushLabel("Installed for MATLAB version "
                                                + installInfo.fullVersion);
                            }
                        }
                        
                        //
                        // has wfdb already been installed and tested for some matlab?
                        //
                        for (Map.Entry<File, InstallerGUI.MatlabInstallation> matlabInstall :
                                 gui.getMatlabInstallations().entrySet())
                        {
                            File path = matlabInstall.getKey();
                            InstallerGUI.MatlabInstallation installInfo = matlabInstall.getValue();
                            
                            if ((installInfo.lastFinishedStep !=
                                 InstallerGUI.InstallationStep.TESTING)
                                || installInfo.hasFailed)
                            {
                                continue;
                            }

                            finishedForSomeMatlab = true;
                            break;
                        }

                        //
                        // move ~/.wfdb-swig-matlab-tmp to ~/.wfdb-swig-matlab
                        //
                        if (!panelThis.tempDirMovedToFinalDir && installedForSomeMatlab)
                        {
                            try
                            {
                                nativeController.moveTempConfigDirToFinalConfigDir();
                                panelThis.tempDirMovedToFinalDir = true;
                            }
                            catch (IOException ex)   // FIXME: test this
                            {
                                panelThis.tempDirMovedToFinalDir = false;
                                logger.severe("Problem renaming directory: " + ex.getMessage()
                                              + "\n");
                                workerPushLabel("Unable to rename temporary directory.");
                                return;
                            }
                        }

                        //
                        // try to update the path for matlabs that have been installed;
                        // this must be done after the temp. dir has been moved to the
                        // final dir., otherwise matlab's addpath() may not succeed
                        // NOTE: we don't do check panelThis.tempDirMovedToFinalDir;
                        //       it doesn't matter very much
                        //
                        for (Map.Entry<File, InstallerGUI.MatlabInstallation> matlabInstall :
                                 gui.getMatlabInstallations().entrySet())
                        {
                            File path = matlabInstall.getKey();
                            InstallerGUI.MatlabInstallation installInfo = matlabInstall.getValue();

                            if ((installInfo.lastFinishedStep ==
                                 InstallerGUI.InstallationStep.INSTALLATION)
                                && (!installInfo.hasFailed))
                            {
                                // update the path variable in this version of matlab;
                                // doing this for every version is perhaps slightly redundant,
                                // but much safer. if it fails, it is not fatal, but we need to
                                // keep track that it did fail.
                                try
                                {
                                    nativeController.updateMatlabPathVariable(path, installInfo);
                                    installInfo.pathUpdated = true;
                                }
                                catch (IOException ex)
                                {
                                    installInfo.pathUpdated = false;
                                    logger.warning("Updating the path for MATLAB in "
                                                   + path
                                                   + " apparently failed (not a fatal error.)\n");
                                }
                                catch (ProcessExecutionException ex)
                                {
                                    installInfo.pathUpdated = false;
                                    logger.warning("Updating the path for MATLAB in "
                                                   + path
                                                   + " apparently failed (not a fatal error.)\n");
                                }
                            }
                        }
                        

                        //
                        // try to test all installed and untested matlabs
                        //
                        for (Map.Entry<File, InstallerGUI.MatlabInstallation> matlabInstall :
                                 gui.getMatlabInstallations().entrySet())
                        {
                            File path = matlabInstall.getKey();
                            InstallerGUI.MatlabInstallation installInfo = matlabInstall.getValue();
                            boolean testingResult;

                            if ((installInfo.lastFinishedStep !=
                                 InstallerGUI.InstallationStep.INSTALLATION)
                                || installInfo.hasFailed)
                            {
                                continue;
                            }

                            workerPushLabel("Running test for MATLAB version "
                                            + installInfo.fullVersion + " ...");

                            try
                            {
                                Pointer<Boolean> pathIsCorrect = new Pointer<Boolean>(false);
                                Pointer<Boolean> wfdbTestIsSuccessful = new Pointer<Boolean>(false);

                                nativeController.runWFDBTest(path, pathIsCorrect, wfdbTestIsSuccessful);
                                testingResult = wfdbTestIsSuccessful.object;
                                if (!pathIsCorrect.object)
                                {
                                    installInfo.pathUpdated = false;
                                }
                            }
                            catch (IOException ex)
                            {
                                logger.severe("Test of WFDB Tools in MATLAB "
                                              + installInfo.fullVersion + " failed: "
                                              + ex.getMessage());
                                testingResult = false;
                            }
                            catch (ProcessExecutionException ex)
                            {
                                logger.severe("Test of WFDB Tools in MATLAB "
                                              + installInfo.fullVersion + " failed: "
                                              + ex.getMessage());
                                testingResult = false;
                            }

                            if (testingResult)
                            {
                                installInfo.lastFinishedStep =
                                    InstallerGUI.InstallationStep.TESTING;
                                finishedForSomeMatlab = true;
                                workerPushLabel("Test for MATLAB version "
                                                + installInfo.fullVersion
                                                + " succeeded.");
                            }
                            else
                            {
                                installInfo.hasFailed = true;
                                justFailedForSomeMatlab = true;
                                workerPushLabel("Test for MATLAB version "
                                                + installInfo.fullVersion
                                                + " was unsuccessful (see log for details).");
                                continue;
                            }
                        }
                        
                        // //
                        // //
                        // //
                        // if (installedForSomeMatlab && !panelThis.userpathVariableUpdated)
                        // {
                        //     workerPushLabel("Updating MATLAB user path...");

                        //     // FIXME: right now, this essentially just randomly chooses
                        //     //        one of the possibly several MATLAB installations
                        //     //        to run this command...
                        //     try
                        //     {
                        //         nativeController.updateMatlabPathVariable(getAnyInstalledMatlabPath());
                        //         panelThis.userpathVariableUpdated = true;
                        //         workerPushLabel("MATLAB user path has been updated.");
                        //     }
                        //     catch (IOException ex)
                        //     {
                        //         // FIXME FILLME
                        //     }
                        //     catch (ProcessExecutionException ex)
                        //     {
                        //         // FIXME FILLME
                        //     }
                        // }
                    }
                    catch (InterruptedException ex)
                    {
                        // this should never happen; if it does, there's a bug.
                        // FIXME: abstract all of this
                        logger.severe("Unknown problem occurred (this is a bug!): "
                                      + ex.toString() + "\n");
                        logger.severe("Stack trace:\n");
                        for (StackTraceElement traceLine : ex.getStackTrace())
                        {
                            logger.severe(traceLine.toString() + "\n");
                        }
                        return;
                    }
                    catch (InvocationTargetException ex)
                    {
                        // this should never happen; if it does, there's a bug.
                        // FIXME: abstract all of this
                        logger.severe("Unknown problem occurred (this is a bug!): "
                                      + ex.toString() + "\n");
                        logger.severe("Stack trace:\n");
                        for (StackTraceElement traceLine : ex.getStackTrace())
                        {
                            logger.severe(traceLine.toString() + "\n");
                        }
                        return;
                    }
                }

                //
                protected void done()
                {
                    if (!panelThis.wfdbFilesCopied || failureCopyingNewArchs)
                    {
                        navUpdateListener.setPreviousButtonEnabled(false);
                        navUpdateListener.setNextButtonEnabled(false);
                        navUpdateListener.setQuitButtonQuiet(true);

                        JOptionPane.showMessageDialog(panelThis.panel,
                                                      "An error occurred while copying the "
                                                      + "WFDB files to your system.\n Please "
                                                      + "view the log for details (click the "
                                                      + "\"View Log\" button.)\n Unfortunately, "
                                                      + "the installation cannot proceed.",
                                                      "Error Copying Files",
                                                      JOptionPane.ERROR_MESSAGE);
                        return;
                    }

                    if (!panelThis.tempDirMovedToFinalDir && installedForSomeMatlab)
                    {
                        navUpdateListener.setPreviousButtonEnabled(false);
                        navUpdateListener.setNextButtonEnabled(false);
                        navUpdateListener.setQuitButtonQuiet(true);

                        JOptionPane.showMessageDialog(panelThis.panel,
                                                      "A fatal I/O error occurred during installation.\n"
                                                      + "This is fairly unexpected; please view the log\n"
                                                      + "for details (click the \"View Log\" button.)\n"
                                                      + "Unfortunately, the installation cannot proceed.",
                                                      "Error Installing Files",
                                                      JOptionPane.ERROR_MESSAGE);
                        return;
                    }



                    //
                    //
                    //
                    if (justFailedForSomeMatlab && installedForSomeMatlab)
                    {
                        // FIXME: put better dialog message here?
                        JOptionPane.showMessageDialog(panelThis.panel,
                                                      "Installation and/or testing of WFDB Tools "
                                                      + "failed for at least one version of "
                                                      + "MATLAB.\n"
                                                      + "If you'd like, you can go back and select "
                                                      + "another version of MATLAB for which to "
                                                      + "install WFDB Tools.\n"
                                                      + "Installation did succeed, though, for "
                                                      + "at least some version(s) of MATLAB, "
                                                      + "so you may safely exit the installer now\n "
                                                      + "and use WFDB Tools with that/those "
                                                      + "versions.\n",
                                                      "Error During Installation",
                                                      JOptionPane.ERROR_MESSAGE);
                        
                        navUpdateListener.setQuitButtonQuiet(true);
                        navUpdateListener.setQuitButtonToOK(true);
                        navUpdateListener.setPreviousButtonEnabled(true);
                        navUpdateListener.setNextButtonEnabled(false);
                        
                        titleLabel.setText("<html><h2>Performing Installation - Done</h2></html>");
                        
                        if (!panelThis.summaryTextDone)
                        {
                            mainVertBox.add(new JLabel(buildSummaryText()));
                            panelThis.summaryTextDone = true;
                        }
                        panel.validate();
                    }
                    else if (justFailedForSomeMatlab && !installedForSomeMatlab)
                    {
                        // FIXME: put better dialog message here?
                        JOptionPane.showMessageDialog(panelThis.panel,
                                                      "Installation of WFDB Tools was not "
                                                      + "successful for any version of MATLAB "
                                                      + "selected.\n"
                                                      + "If you'd like, you can go back and select "
                                                      + "another version of MATLAB for which to "
                                                      + "install WFDB Tools.\n",
                                                      "Error During Installation",
                                                      JOptionPane.ERROR_MESSAGE);
                        
                        navUpdateListener.setQuitButtonQuiet(false);
                        navUpdateListener.setQuitButtonToOK(false);
                        navUpdateListener.setPreviousButtonEnabled(true);
                        navUpdateListener.setNextButtonEnabled(false);

                        titleLabel.setText("<html><h2>Performing Installation</h2></html>");
                        
                        if (!panelThis.summaryTextDone)
                        {
                            mainVertBox.add(new JLabel(buildSummaryText()));
                            panelThis.summaryTextDone = true;
                        }
                        panel.validate();
                    }
                    else if (!installedForSomeMatlab)
                    {
                        // didn't just fail, and still not installed for any matlab,
                        // i.e. there were no new matlabs to try when this panel was loaded
                        
                        navUpdateListener.setQuitButtonQuiet(false);
                        navUpdateListener.setQuitButtonToOK(false);
                        navUpdateListener.setPreviousButtonEnabled(true);
                        navUpdateListener.setNextButtonEnabled(false);

                        titleLabel.setText("<html><h2>Performing Installation</h2></html>");

                        if (!panelThis.summaryTextDone)
                        {
                            mainVertBox.add(new JLabel(buildSummaryText()));
                            panelThis.summaryTextDone = true;
                        }
                        panel.validate();
                    }
                    else
                    {
                        navUpdateListener.setQuitButtonQuiet(true);
                        navUpdateListener.setQuitButtonToOK(true);
                        navUpdateListener.setPreviousButtonEnabled(true);
                        navUpdateListener.setNextButtonEnabled(false);

                        titleLabel.setText("<html><h2>Performing Installation - Done</h2></html>");

                        if (!panelThis.summaryTextDone)
                        {
                            mainVertBox.add(new JLabel(buildSummaryText()));
                            panelThis.summaryTextDone = true;
                        }
                        panel.validate();
                    }
                }
            }).execute();
    }

    
    /*
     * this essentially figures out whether we are going to attempt to install
     * new packages, so that we can prepare the pane without any initial
     * flicking (clearing -> redrawing)
     *
     * FIXME: this does a lot of double-duty with postload()
     */
    public void preload()
    {
        boolean newToInstall;
        boolean someHasFinished;


        //
        this.mainVertBox.removeAll();
        this.summaryTextDone = false;
        titleLabel.setText("<html><h2>Performing Installation</h2></html>");

        //
        if (!this.wfdbFilesCopied
            || !this.tempDirMovedToFinalDir
            ) // || !this.userpathVariableUpdated)
        {
            return;
        }

        //
        newToInstall = false;
        someHasFinished = false;
        for (Map.Entry<File, InstallerGUI.MatlabInstallation> matlabInstall :
                 gui.getMatlabInstallations().entrySet())
        {
            InstallerGUI.MatlabInstallation installInfo =
                matlabInstall.getValue();

            switch (installInfo.lastFinishedStep)
            {
            case VERIFICATION:
                if (!installInfo.hasFailed)
                {
                    newToInstall = true;
                }
                break;

            case INSTALLATION:
                if (!installInfo.hasFailed)
                {
                    newToInstall = true;
                }
                someHasFinished = true;
                break;

            case TESTING:
                someHasFinished = true;
                break;
            }
        }
        if (newToInstall)
        {
            return;
        }

        if (someHasFinished)
        {
            titleLabel.setText("<html><h2>Performing Installation - Done</h2></html>");
        }

        mainVertBox.add(new JLabel(buildSummaryText()));
        this.summaryTextDone = true;
    }


    /*
     *
     */
    public boolean unload()
    {
        return true;
    }


    /*
     *
     */
    private void workerPushLabel(final String labelText)
        throws InterruptedException,
               InvocationTargetException
    {
        Runnable runnable;


        runnable = new Runnable() {
                public void run()
                {
                    mainVertBox.add(new JLabel(labelText));
                    panel.validate();
                }
            };
        
        EventQueue.invokeAndWait(runnable);
    }
}

