/*
 *
 */


/*
 *
 */
package installer.common;


/*
 *
 */
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;

import installer.common.GUIWorker;


/*
 *
 */
public class SearchForMatlabsPanel extends InstallerPanel
{
    /*
     *
     */
    private boolean hasBeenLoadedBefore;


    /*
     * GUI pieces
     */
    private Box centerBox;
    private Box choosePathBox;
    private Box matlabPathsBox;
    private Box messageBox;
    private JTable matlabPathsTable;
    private JTextField matlabPathTextField;
    private JButton addMatlabPathButton;
    private JButton chooseMatlabPathButton;
    private JLabel currentMessage;


    /*
     *
     */
    private MatlabPathsTableModel matlabPathsTableModel;
    private TableModelListener matlabPathsTableModelListener;


    /*
     * passed in during postload(); remove during unload()
     */
    private InstallerWindow.NavUpdateListener navUpdateListener;
    private Controller nativeController;


    /*
     *
     */
    private static class MatlabPathsTableModel extends DefaultTableModel
    {
        //
        public boolean isCellEditable(int row,
                                      int column)
        {
            return (column == 2);
        }

        //
        public Class getColumnClass(int column)
        {
            return (column < 2) ? String.class : Boolean.class;
        }
    }


    /*
     *
     */
    public SearchForMatlabsPanel()
    {
        //
        this.hasBeenLoadedBefore = false;
        this.gui = null;
        this.matlabPathsTableModelListener = null;
        
        //
        this.navUpdateListener = null;
        this.nativeController = null;

        // FIXMEFIXME: test this!
        // this.matlabPathsTableModelListener = new TableModelListener() {
        //         public void tableChanged(TableModelEvent event)
        //         {
        //             if (event.getColumn() == 2)
        //             {
        //                 Boolean value = matlabPathsTableModel.getValueAt(event.getFirstRow(), 2);
                        
        //                 numRowsChecked += value.booleanValue() ? 1 : -1;
        //                 if (numRowsChecked == 0)
        //                 {
                            
        //             }
        //         }
        //     };

        //
        buildPanelGUI();
    }


    /*
     * FIXME: a bit slow -- not that it matters much
     */
    private boolean allMatlabPathsUnchecked()
    {
        int i;
        
        
        //
        for (i = 0; i < this.matlabPathsTableModel.getRowCount(); ++i)
        {
            if (((Boolean)this.matlabPathsTableModel.getValueAt(i, 2)).booleanValue())
            {
                return false;
            }
        }

        //
        return true;
    }


    /*
     *
     */
    private void attemptToAddMatlabPath(String matlabPathString)
    {
        final File matlabPath;


        //
        if (matlabPathString.length() == 0)
        {
            JOptionPane.showMessageDialog(this.panel,
                                          "Please provide a valid directory name.",
                                          "Invalid Directory Name",
                                          JOptionPane.ERROR_MESSAGE);
            return;
        }

        //
        this.navUpdateListener.setPreviousButtonEnabled(false);
        this.navUpdateListener.setNextButtonEnabled(false);

        //
        matlabPath = new File(matlabPathString);

        //
        currentMessage.setText("<html><h3>Attempting to verify MATLAB installation in "
                               + matlabPath.getAbsolutePath() + "</h3></html>");
        this.panel.repaint();

        enableMatlabPathsBox(false);

        //
        (new GUIWorker() {
                //
                // private String fullVersion = null;
                private InstallerGUI.MatlabInstallation matlabInst = null;

                //
                protected void doInBackground()
                {
                    matlabInst = nativeController.verifyMatlabPathAndGetVersion(matlabPath);
                }

                //
                protected void done()
                {
                    if (matlabInst == null)
                    {
                        JOptionPane.showMessageDialog(panel,
                                                      "Unable to verify MATLAB installation in: \n"
                                                      + matlabPath.getAbsolutePath(),
                                                      "Unverifiable MATLAB Installation",
                                                      JOptionPane.ERROR_MESSAGE);

                        currentMessage.setText("<html><h3>" + matlabPath.getAbsolutePath()
                                               + " does not appear to contain a valid, useable "
                                               + "installtion of MATLAB.</h3></html>");
                    }
                    else
                    {
                        // FIXMEFIXME: also make sure this install is not already there!!!
                        // FIXME: duplicate code
                        // FIXMEFIXME: the first entry should be the File itself, not a String
                        Object[] row = { matlabPath.getAbsolutePath(),
                                         matlabInst.fullVersion,
                                         new Boolean(true) };
                        
                        matlabInst.lastFinishedStep =
                            InstallerGUI.InstallationStep.VERIFICATION;
                        matlabPathsTableModel.addRow(row);
                        // gui.giveMatlabPathAndVersion(matlabPath, fullVersion);
                        gui.giveMatlabPathAndVersion(matlabPath, matlabInst);

                        currentMessage.setText("<html><h3>MATLAB installation in "
                                               + matlabPath.getAbsolutePath()
                                               + " verified.</h3></html>");
                    }

                    enableMatlabPathsBox(true);

                    panel.repaint();
                    navUpdateListener.setPreviousButtonEnabled(true);
                    navUpdateListener.setNextButtonEnabled(true);                    
                }
            }).execute();
    }

    
    /*
     *
     */
    private void buildPanelGUI()
    {
        Dimension d;
        JLabel titleLabel;


        //
        this.matlabPathsTableModel = new MatlabPathsTableModel();
        this.matlabPathsTableModel.addColumn("MATLAB Path");
        this.matlabPathsTableModel.addColumn("MATLAB Version");
        this.matlabPathsTableModel.addColumn("Install");
        this.matlabPathsTable = new JTable(this.matlabPathsTableModel);
        this.matlabPathsTable.setMinimumSize(new Dimension(100, 100)); // FIXME: this is a hack!

        //
        this.matlabPathTextField = new JTextField(64);

        // FIXME: this is a hack!
        d = new Dimension((int)this.matlabPathTextField.getMaximumSize().getWidth(),
                          (int)(this.matlabPathTextField.getMinimumSize().getHeight() * 1.5));
        this.matlabPathTextField.setMaximumSize(d);

        //
        this.chooseMatlabPathButton = new JButton("Choose...");
        this.addMatlabPathButton = new JButton("Add");

        //
        this.choosePathBox = Box.createHorizontalBox();
        this.choosePathBox.add(this.matlabPathTextField);
        this.choosePathBox.add(this.chooseMatlabPathButton);
        this.choosePathBox.add(this.addMatlabPathButton);

        //
        this.matlabPathsBox = Box.createVerticalBox();
        this.matlabPathsBox.add(Box.createVerticalGlue());
        this.matlabPathsBox.add(new JScrollPane(this.matlabPathsTable));
        this.matlabPathsBox.add(this.choosePathBox);
        this.matlabPathsBox.add(Box.createVerticalGlue());


        //
        this.currentMessage = new JLabel("<html></html>");
        this.messageBox = Box.createVerticalBox();
        this.messageBox.add(this.currentMessage);

        //
        this.centerBox = Box.createHorizontalBox();
        this.centerBox.add(this.matlabPathsBox);
        this.centerBox.add(this.messageBox);

        //
        this.chooseMatlabPathButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent event)
                {
                    showAndHandleMatlabPathChooser();
                }
            });

        //
        this.addMatlabPathButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent event)
                {
                    attemptToAddMatlabPath(matlabPathTextField.getText());
                }
            });

        //
        this.panel.setLayout(new BorderLayout());

        //
        titleLabel = new JLabel("<html><h2>Choose MATLAB Installations</h2></html>");
        titleLabel.setHorizontalAlignment(JLabel.CENTER);
        this.panel.add(titleLabel, BorderLayout.NORTH);

        //
        this.panel.add(this.centerBox, BorderLayout.CENTER);

        // FIXME: this is a hack! we just want to split the panel equally!
        this.matlabPathsBox.setPreferredSize(new Dimension(300, 400));
        this.messageBox.setPreferredSize(new Dimension(300, 400));

        //
        this.panel.validate();
    }


    /*
     * this is required because, contrary to intuition (mine, at least), calling
     * setEnabled() on a Box (or a JPanel, for that matter) does not affect its
     * contained children.
     */
    private void enableMatlabPathsBox(boolean enabled)
    {
        this.matlabPathsTable.setEnabled(enabled);
        this.matlabPathTextField.setEnabled(enabled);
        this.chooseMatlabPathButton.setEnabled(enabled);
        this.addMatlabPathButton.setEnabled(enabled);
    }


    /*
     *
     */
    public void postload(final Controller nativeController,
                         final InstallerWindow.NavUpdateListener navUpdateListener)
    {
        //
        this.navUpdateListener = navUpdateListener;
        this.nativeController = nativeController;

        //
        this.navUpdateListener.setPreviousButtonEnabled(false);
        this.navUpdateListener.setNextButtonEnabled(false);
        this.panel.validate();
        this.panel.repaint();

        // FIXME: this setup is unnecessary -- it's only to access the navUpdateListener
        // FIXMEFIXME: the first column should be File, not String
        //
        this.matlabPathsTableModelListener = new TableModelListener() {
                public void tableChanged(TableModelEvent event)
                {
                    if (event.getColumn() == 2)
                    {
                        String matlabPath =
                            (String) matlabPathsTableModel.getValueAt(event.getFirstRow(), 0);
                        Boolean selected =
                            (Boolean) matlabPathsTableModel.getValueAt(event.getFirstRow(), 2);
                        InstallerGUI.MatlabInstallation matlabInstall =
                            gui.getMatlabInstallations().get(new File(matlabPath));

                        //
                        if (selected.booleanValue())
                        {
                            gui.getMatlabInstallations().get(new File(matlabPath)).lastFinishedStep =
                                InstallerGUI.InstallationStep.VERIFICATION;
                        }
                        else
                        {
                            gui.getMatlabInstallations().get(new File(matlabPath)).lastFinishedStep =
                                InstallerGUI.InstallationStep.UNSELECTED;
                        }

                        //
                        if (allMatlabPathsUnchecked())
                        {
                            navUpdateListener.setNextButtonEnabled(false);
                        }
                        else
                        {
                            navUpdateListener.setNextButtonEnabled(true);
                        }
                    }
                }
            };
        this.matlabPathsTableModel.addTableModelListener(this.matlabPathsTableModelListener);

        //
        if (!this.hasBeenLoadedBefore)
        {
            this.hasBeenLoadedBefore = true;

            enableMatlabPathsBox(false);

            this.currentMessage.setText("<html><h3>Searching for MATLAB installations, "
                                        + "please wait...</h3></html>");
            this.panel.repaint();
            
            (new GUIWorker() {
                    //
                    private Map<File, InstallerGUI.MatlabInstallation> matlabPathsToVersions = null;

                    //
                    private boolean configGatheringFailed = false;

                    //
                    protected void doInBackground()
                    {
                        List<String> configInfoErrors = new LinkedList<String>();
                        boolean r;


                        //
                        this.configGatheringFailed = false;
                        
                        //
                        r = nativeController.gatherSystemConfigurationInformation(configInfoErrors);
                        if (!r)
                        {
                            for (String errorString : configInfoErrors)
                            {
                                // FIXME: keep logger as a member var.
                                Logger.getLogger("controller").severe(errorString + "\n");
                            }

                            this.configGatheringFailed = true;
                            return;
                        }

                        //
                        this.matlabPathsToVersions = nativeController.findMatlabPathsAndVersions();
                    }

                    //
                    protected void done()
                    {
                        int numMatlabs;


                        //
                        if (configGatheringFailed)
                        {
                            JOptionPane.showMessageDialog(panel,
                                                          "An error occurred: the installer was "
                                                          + "unable to gather all of the \n"
                                                          + "necessary configuration information "
                                                          + "(see the log for details.)\n "
                                                          + "Unfortunately, this means installation "
                                                          + "cannot proceed.\n",
                                                          "Unable to Proceed",
                                                          JOptionPane.ERROR_MESSAGE);
                            navUpdateListener.setQuitButtonQuiet(true);
                            currentMessage.setText("<html><h3>Unable to gather system "
                                                   + "configuration information (please check "
                                                   + "the log.)</h3></html>");
                            enableMatlabPathsBox(false);
                            panel.repaint();
                            return;
                        }

                        //
                        numMatlabs = 0;
                        for (Map.Entry<File, InstallerGUI.MatlabInstallation> pathAndVersion :
                                 matlabPathsToVersions.entrySet())
                        {
                            Object[] row = { pathAndVersion.getKey().getAbsolutePath(),
                                             pathAndVersion.getValue().fullVersion,
                                             new Boolean(true) };

                            pathAndVersion.getValue().lastFinishedStep =
                                InstallerGUI.InstallationStep.VERIFICATION;
                            matlabPathsTableModel.addRow(row);
                            gui.giveMatlabPathAndVersion(pathAndVersion.getKey(),
                                                         pathAndVersion.getValue());
                            ++numMatlabs;
                        }

                        //
                        if (numMatlabs == 0)
                        {
                            currentMessage.setText("<html><h3>No installations "
                                                   + "of MATLAB found!</h3></html>");
                            navUpdateListener.setNextButtonEnabled(false);
                            // FIXMEFIXME: show an error dialog
                        }
                        else if (numMatlabs == 1)
                        {
                            currentMessage.setText("<html><h3>One installation of "
                                                   + "MATLAB found.</h3></html>");
                            navUpdateListener.setNextButtonEnabled(true);
                        }
                        else
                        {
                            currentMessage.setText("<html><h3>" + numMatlabs + " installations of "
                                                   + "MATLAB found.</h3></html>");
                            navUpdateListener.setNextButtonEnabled(true);
                        }

                        //
                        enableMatlabPathsBox(true);
                        navUpdateListener.setPreviousButtonEnabled(true);
                        panel.repaint();
                    }
                }).execute();
        }
        else
        {
            // // fill the table with known (validated), but yet
            // // wfdb-uninstalled-for paths of matlab
            // for (Map.Entry<File, String> pathAndVersion : gui.getUninstalledMatlabs().entrySet())
            // {
            //     Object[] row = { pathAndVersion.getKey().getAbsolutePath(),
            //                      pathAndVersion.getValue(),
            //                      new Boolean(true) };
                
            //     this.matlabPathsTableModel.addRow(row);
            // }

            // fill the table with known (validated), but yet
            // wfdb-uninstalled-for paths of matlab
            for (Map.Entry<File, InstallerGUI.MatlabInstallation> matlabInstall :
                     gui.getMatlabInstallations().entrySet())
            {
                File path = matlabInstall.getKey();
                InstallerGUI.MatlabInstallation installInfo = matlabInstall.getValue();

                if ((installInfo.lastFinishedStep ==
                     InstallerGUI.InstallationStep.VERIFICATION)
                    && !installInfo.hasFailed)
                {
                    Object[] row = { path.getAbsolutePath(),
                                     installInfo.fullVersion,
                                     new Boolean(true) };

                    this.matlabPathsTableModel.addRow(row);
                }
            }

            //
            this.navUpdateListener.setPreviousButtonEnabled(true);
            this.navUpdateListener.setNextButtonEnabled(true);
        }
    }


    /*
     *
     */
    public void preload()
    {
    }


    /*
     *
     */
    private void showAndHandleMatlabPathChooser()
    {
        JFileChooser fileChooser;
        int result;


        //
        fileChooser = new JFileChooser();
        fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        
        //
        result = fileChooser.showOpenDialog(this.panel);
        if (result == JFileChooser.APPROVE_OPTION)
        {
            this.matlabPathTextField.setText(fileChooser.getSelectedFile().getAbsolutePath());
        }
    }


    /*
     *
     */
    public boolean unload()
    {
        int i, numRows;


        //
        this.navUpdateListener = null;
        this.nativeController = null;

        //
        this.matlabPathsTableModel.setRowCount(0);
        this.matlabPathsTableModel.removeTableModelListener(this.matlabPathsTableModelListener);
        this.matlabPathsTableModelListener = null;

        //
        return true;
    }
}