/*
 *
 */


/*
 *
 */
import java.io.File;
import java.io.IOException;
import java.util.Vector;
import java.util.logging.Logger;
import installer.common.MatlabRunner;
import installer.common.ProcessExecutionException;
import installer.common.ProcessOutputAnalyzer;
import installer.common.ProcessRunner;


/*
 * this is a singleton
 */
public class MacOSXMatlabRunner extends MatlabRunner
{
    /*
     *
     */
    private static final MacOSXMatlabRunner runner = new MacOSXMatlabRunner();


    /*
     *
     */
    private MacOSXMatlabRunner()
    {
    }


    /*
     *
     */
    public static MacOSXMatlabRunner get()
    {
        return runner;
    }


    /*
     * this doesn't need all the machinery of ProcessRunner;
     * stdout and stdin are ignored -- it just launches matlab
     * with its gui
     *
     * FIXME: but, should figure out a way to tell when this has failed
     */
    public void launch(File matlabPath,
                       String matlabCommand,
                       File workingDirectory)
        throws IOException
    {
        String[] commandLine;
        File matlabBinaryPath;

        
        //
        matlabBinaryPath = new File(new File(matlabPath, "bin"), "matlab");
        if (!matlabBinaryPath.exists())
        {
            throw new IOException("Expected MATLAB binary "
                                  + matlabBinaryPath.getAbsolutePath() + " "
                                  + "does not exist");
        }

        //
        commandLine =
            new String[] { matlabBinaryPath.getAbsolutePath(),
                           "-desktop",
                           "-r", matlabCommand };
        Runtime.getRuntime().exec(commandLine, null, workingDirectory);
    }


    /*
     * FIXME: this is identical to the linux version
     */
    public <ResultType> ResultType
        run(File matlabPath,
            String matlabCommand,
            File logFile,
            ProcessOutputAnalyzer<ResultType> outputAnalyzer)
        throws IOException,
               ProcessExecutionException
    {
        Logger logger;
        File matlabBinaryPath;
        Vector<String> commandLine;
        ProcessRunner.Result<ResultType> matlabResult;


        //
        logger = Logger.getLogger("controller");

        //
        matlabBinaryPath = new File(new File(matlabPath, "bin"), "matlab");
        if (!matlabBinaryPath.exists())
        {
            throw new IOException("Expected MATLAB binary "
                                  + matlabBinaryPath.getAbsolutePath() + " "
                                  + "does not exist");
        }

        //
        commandLine = new Vector<String>();
        commandLine.add(matlabBinaryPath.getAbsolutePath());
        commandLine.add("-nodesktop");
        commandLine.add("-nodisplay");
        commandLine.add("-nosplash");
        if (logFile != null)
        {
            commandLine.add("-logfile");
            commandLine.add(logFile.getAbsolutePath());
        }
        commandLine.add("-r");
        commandLine.add(matlabCommand);

        //
        logger.info("Running MATLAB (" + matlabBinaryPath.getAbsolutePath() + ") with "
                    + ((logFile != null)
                       ? ("log file " + logFile.getAbsolutePath() + " and ")
                       : "")
                    + "command: " + matlabCommand + " ...\n");

        //
        matlabResult = ProcessRunner.run(outputAnalyzer, commandLine.toArray(new String[0]));

        //
        logger.info("MATLAB finished running.\n");

        //
        return matlabResult.outputResult;
    }
}

