/*
 *
 */


/*
 *
 */
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.util.Vector;
import java.util.logging.Logger;
import installer.common.MatlabRunner;
import installer.common.ProcessExecutionException;
import installer.common.ProcessOutputAnalyzer;
import installer.common.ProcessRunner;


/*
 * this is a singleton
 */
public class WindowsArchSubdirMatlabRunner extends MatlabRunner
{
    /*
     *
     */
    private static final WindowsArchSubdirMatlabRunner runner =
        new WindowsArchSubdirMatlabRunner();


    /*
     *
     */
    private WindowsArchSubdirMatlabRunner()
    {
    }


    /*
     *
     */
    public static WindowsArchSubdirMatlabRunner get()
    {
        return runner;
    }


    /*
     *
     */
    public void launch(File matlabPath,
                       String matlabCommand,
                       File workingDirectory)
        throws IOException
    {
        String[] commandLine;
        File[] matlabBinaryPaths;
        File matlabBinaryPath;


        //
        matlabBinaryPaths = new File[] {
            new File(new File(new File(matlabPath, "bin"), "win32"), "matlab.exe"),
            new File(new File(new File(matlabPath, "bin"), "win64"), "matlab.exe")
        };
        matlabBinaryPath = null;
        for (File path : matlabBinaryPaths)
        {
            if (path.exists())
            {
                matlabBinaryPath = path;
                break;
            }
        }
        if (matlabBinaryPath == null)
        {
            throw new IOException("Could not find architecture-specific MATLAB binary in "
                                  + matlabPath.getAbsolutePath());
        }

        //
        commandLine =
            new String[] { "CMD.EXE", "/C", "START", "/WAIT",
                           matlabBinaryPath.getAbsolutePath(),
                           //"MATLAB",
                           matlabBinaryPath.getAbsolutePath(), "-r",
                           matlabCommand };
        Runtime.getRuntime().exec(commandLine, null, workingDirectory);
    }


    /*
     *
     */
    public <ResultType> ResultType
        run(File matlabPath,
            String matlabCommand,
            File logFile,
            ProcessOutputAnalyzer<ResultType> outputAnalyzer)
        throws IOException,
               ProcessExecutionException
    {
        Logger logger;
        File[] matlabBinaryPaths;
        File matlabBinaryPath;
        Vector<String> commandLine;
        String[] commandLineArray;
        boolean logFileIsTemp;
        ResultType result;
        BufferedReader logFileReader;
        int i;


        //
        logger = Logger.getLogger("controller");

        //
        matlabBinaryPaths = new File[] {
            new File(new File(new File(matlabPath, "bin"), "win32"), "matlab.exe"),
            new File(new File(new File(matlabPath, "bin"), "win64"), "matlab.exe")
        };
        matlabBinaryPath = null;
        for (File path : matlabBinaryPaths)
        {
            if (path.exists())
            {
                matlabBinaryPath = path;
                break;
            }
        }
        if (matlabBinaryPath == null)
        {
            throw new IOException("Could not find architecture-specific MATLAB binary in "
                                  + matlabPath.getAbsolutePath());
        }

        //
        logFileIsTemp = false;
        if (logFile == null && outputAnalyzer != null)
        {
            logFileIsTemp = true;
            logFile = File.createTempFile("wsm", null);
        }

        //
        commandLine = new Vector<String>();
        commandLine.add("cmd.exe");
        commandLine.add("/C");
        commandLine.add("START");
        commandLine.add("/WAIT");
        //commandLine.add("MATLAB");
        commandLine.add(matlabBinaryPath.getAbsolutePath());        
        commandLine.add(matlabBinaryPath.getAbsolutePath());
        commandLine.add("-nodesktop");
        commandLine.add("-nosplash");
        commandLine.add("-automation");
        if (logFile != null)
        {
            commandLine.add("-logfile");
            commandLine.add(logFile.getAbsolutePath());
        }
        commandLine.add("-r");
        commandLine.add(matlabCommand);
        commandLineArray = commandLine.toArray(new String[0]);

        //
        logger.info("Running MATLAB (" + matlabBinaryPath.getAbsolutePath() + ") with "
                    + ((logFile != null)
                       ? ("log file " + logFile.getAbsolutePath() + " and ")
                       : "")
                    + "command: " + matlabCommand + " ...\n");
        
        //
        ProcessRunner.run(null, commandLineArray);

        //
        try
        {
            if (outputAnalyzer != null)
            {
                logFileReader =
                    new BufferedReader(new InputStreamReader(new FileInputStream(logFile)));
                try
                {
                    result = outputAnalyzer.analyzeOutput(logFileReader);
                }
                catch (ProcessOutputAnalyzer.UnexpectedOutputException ex)
                {
                    // this essentially mimics what ProcessRunner does
                    throw new ProcessExecutionException(commandLineArray, ex.getMessage());
                }
            }
            else
            {
                result = null;
            }
        }
        finally
        {
            if (logFile != null && logFileIsTemp && logFile.exists())
            {
                if (!logFile.delete())
                {
                    logger.warning("Unable to delete temporary file: "
                                   + logFile.getAbsolutePath() + " "
                                   + "(not a fatal error.)\n");
                }
            }
        }

        //
        return result;
    }
}