/*
 *
 */


/*
 *
 */
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.util.Vector;
import java.util.logging.Logger;
import installer.common.MatlabRunner;
import installer.common.ProcessExecutionException;
import installer.common.ProcessOutputAnalyzer;
import installer.common.ProcessRunner;


/*
 * this is a singleton
 */
public class WindowsCommandLineMatlabRunner extends MatlabRunner
{
    /*
     *
     */
    private static final WindowsCommandLineMatlabRunner runner =
        new WindowsCommandLineMatlabRunner();


    /*
     *
     */
    private WindowsCommandLineMatlabRunner()
    {
    }


    /*
     *
     */
    public static WindowsCommandLineMatlabRunner get()
    {
        return runner;
    }

    
    /*
     * this doesn't need all the machinery of ProcessRunner;
     * it just launches matlab with its gui
     *
     * FIXME: but, should figure out a way to tell when this has failed
     */
    public void launch(File matlabPath,
                       String matlabCommand,
                       File workingDirectory)
        throws IOException
    {
        String[] commandLine;
        File matlabBinaryPath;


        //
        matlabBinaryPath = new File(new File(matlabPath, "bin"), "matlab.exe");
        if (!matlabBinaryPath.exists())
        {
            throw new IOException("Expected MATLAB binary "
                                  + matlabBinaryPath.getAbsolutePath() + " "
                                  + "does not exist");
        }

        //
        commandLine =
            new String[] { matlabBinaryPath.getAbsolutePath(),
                           "-desktop",
                           "-r", matlabCommand };
        Runtime.getRuntime().exec(commandLine, null, workingDirectory);
    }


    /*
     *
     */
    public <ResultType> ResultType
        run(File matlabPath,
            String matlabCommand,
            File logFile,
            ProcessOutputAnalyzer<ResultType> outputAnalyzer)
        throws IOException,
               ProcessExecutionException
    {
        Logger logger;
        File matlabBinaryPath;
        Vector<String> commandLine;
        String[] commandLineArray;
        boolean logFileIsTemp;
        ResultType result;
        BufferedReader logFileReader;


        //
        logger = Logger.getLogger("controller");

        //
        matlabBinaryPath = new File(new File(matlabPath, "bin"), "matlab.exe");
        if (!matlabBinaryPath.exists())
        {
            throw new IOException("Expected MATLAB binary "
                                  + matlabBinaryPath.getAbsolutePath() + " "
                                  + "does not exist");
        }

        //
        logFileIsTemp = false;
        if (logFile == null && outputAnalyzer != null)
        {
            logFileIsTemp = true;
            logFile = File.createTempFile("wsm", null);
        }

        //
        commandLine = new Vector<String>();
        commandLine.add(matlabBinaryPath.getAbsolutePath());
        commandLine.add("-nodesktop");
        commandLine.add("-nosplash");
        commandLine.add("-wait");
        commandLine.add("-automation");
        if (logFile != null)
        {
            commandLine.add("-logfile");
            commandLine.add(logFile.getAbsolutePath());
        }
        commandLine.add("-r");
        commandLine.add(matlabCommand);
        commandLineArray = commandLine.toArray(new String[0]);

        //
        logger.info("Running MATLAB (" + matlabBinaryPath.getAbsolutePath() + ") with "
                    + ((logFile != null)
                       ? ("log file " + logFile.getAbsolutePath() + " and ")
                       : "")
                    + "command: " + matlabCommand + " ...\n");

        //
        ProcessRunner.run(null, commandLineArray);
        
        //
        try
        {
            if (outputAnalyzer != null)
            {
                logFileReader =
                    new BufferedReader(new InputStreamReader(new FileInputStream(logFile)));
                try
                {
                    result = outputAnalyzer.analyzeOutput(logFileReader);
                }
                catch (ProcessOutputAnalyzer.UnexpectedOutputException ex)
                {
                    // this essentially mimics what ProcessRunner does
                    throw new ProcessExecutionException(commandLineArray, ex.getMessage());
                }
            }
            else
            {
                result = null;
            }
        }
        finally
        {
            if (logFile != null && logFileIsTemp && logFile.exists())
            {
                if (!logFile.delete())
                {
                    logger.warning("Unable to delete temporary file: "
                                   + logFile.getAbsolutePath() + " "
                                   + "(not a fatal error.)\n");
                }
            }
        }

        //
        return result;
    }
}

