/*
 * NOTE: the original wfdbdesc app., for variable-layout multi-segment
 *       records, will explicitly print out which data apply to the
 *       entire record, and which data apply only (necessarily) to the
 *       first segment; here, we do not get such a message, and it must
 *       be inferred from the RecordDescriptor.multisegmentType field.
 *
 *       this also means that RecordDescriptor.numSegmentSampleIntervals
 *       and RecordDescriptor.segmentLength may or may not contain valid
 *       data.
 */


/*
 *
 */
import java.io.File;
import java.lang.Class;
import java.lang.ClassLoader;
import java.lang.reflect.InvocationTargetException;
import java.util.*;
import wfdb.*;


/*
 * FIXME: check long's vs. int's
 */
public class wfdbdescMatlab
{
    /*
     *
     */
    static
    {
        try
        {
            Class nativeLibLoader
                = ClassLoader.getSystemClassLoader().loadClass("NativeLibLoader");
            nativeLibLoader.getMethod("loadLibraries", null).invoke(null, null);
        }
        catch (ClassNotFoundException ex)
        {
        }
        catch (NoSuchMethodException ex)
        {
        }
        catch (IllegalAccessException ex)
        {
        }
        catch (InvocationTargetException ex)
        {
        }
    }


    /*
     * this is used essentially as a struct
     */
    public static class GroupDescriptor
    {
	public String filename;
	public int storageFormat;
	public int blockSize;
	public SignalDescriptor[] signals;
    }


    /*
     * this is used essentially as a struct
     */
    public static class RecordDescriptor
    {
	public GroupDescriptor[] groups;
	public String length;
	public boolean lengthSpecified;
	public String multisegmentType;
	public String notes;
	public int numSampleIntervals;
	public int numSegmentSampleIntervals;
	public double samplingFrequency;
	public String segmentLength;
	public String startingTime;

	/*
	 *
	 */
	public RecordDescriptor()
	{
	    groups = null;
	    length = "";
	    lengthSpecified = false;
	    multisegmentType = "";
	    notes = "";
	    numSampleIntervals = 0;
	    numSegmentSampleIntervals = 0;
	    samplingFrequency = 0.0;
	    segmentLength = null;
	    startingTime = "";
	}
    }


    /*
     * this is used essentially as a struct
     */
    public static class SignalDescriptor
    {
	public long groupNumber;
	public int signalNumber;
	public String description;
	public double gain;
	public String units;
	public int initVal;
	public int samplesPerFrame;
	public int adcResolution;
	public int adcZero;
	public int baseline;
	public int checksum;
    }

    
    /*
     * create the GroupDescriptor's using WFDB's Siginfo array.
     *
     * it is assumed that the group numbers take on the values
     * 0 through n-1 (n being the number of groups). 
     */
    protected static GroupDescriptor[] createGroupDescriptors(WFDB_SiginfoArray siarray,
							      int numSignals)
    {
	GroupDescriptor[] groupDescs;
	int[] groupSignalCounts; // number of signals per group
	long maxGroup;
	int i;


	//
	for (i = 0, maxGroup = -1; i < numSignals; ++i)
	{
	    WFDB_Siginfo si = siarray.getitem(i);

	    if (si.getGroup() > maxGroup)
	    {
		maxGroup = si.getGroup();
	    }
	}

	//
	groupDescs = new GroupDescriptor[(int)maxGroup + 1];
	groupSignalCounts = new int[(int)maxGroup + 1];
	for (i = 0; i < groupSignalCounts.length; ++i)
	{
	    groupSignalCounts[i] = 0;
	}

	//
	for (i = 0; i < numSignals; ++i)
	{
	    WFDB_Siginfo si = siarray.getitem(i);
	    long groupNum = si.getGroup();
	    
	    if (groupDescs[(int)groupNum] != null)
	    {
		++groupSignalCounts[(int)groupNum];
		continue;
	    }

	    groupDescs[(int)groupNum] = new GroupDescriptor();
	    groupDescs[(int)groupNum].filename = si.getFname();
	    groupDescs[(int)groupNum].storageFormat = si.getFmt();
	    groupDescs[(int)groupNum].blockSize = si.getBsize();
	    ++groupSignalCounts[(int)groupNum];
	}

	//
	for (i = 0; i < groupDescs.length; ++i)
	{
	    groupDescs[i].signals = new SignalDescriptor[groupSignalCounts[i]];
	}

	//
	for (i = 0; i < numSignals; ++i)
	{
	    WFDB_Siginfo si = siarray.getitem(i);
	    long groupNum = si.getGroup();
	    int signalIndex;
	    
	    signalIndex =
		groupDescs[(int)groupNum].signals.length - groupSignalCounts[(int)groupNum];
	    groupDescs[(int)groupNum].signals[signalIndex] = createSignalDescriptor(si, i);
	    --groupSignalCounts[(int)groupNum];
	}

	return groupDescs;
    }


    /*
     *
     */
    public static SignalDescriptor createSignalDescriptor(WFDB_Siginfo si,
							  int signalNumber)
    {
	SignalDescriptor desc;

	
	desc = new SignalDescriptor();
	desc.groupNumber = si.getGroup(); // FIXME: somewhat redundant, currently
	desc.signalNumber = signalNumber;
	desc.description = si.getDesc(); // FIXME: trim this?

	if (si.getGain() == 0.0) {
	    desc.gain = wfdb.WFDB_DEFGAIN;
	} else {
	    desc.gain = si.getGain();
	}

	desc.units = si.getUnits();
   desc.units = "adu/" + (desc.units != null ? desc.units : "mV");
	desc.initVal = si.getInitval();
	desc.samplesPerFrame = si.getSpf();
	desc.adcResolution = si.getAdcres();
	desc.adcZero = si.getAdczero();
	desc.baseline = si.getBaseline();
	desc.checksum = si.getCksum();

	return desc;
    }


    /*
     * if readable is true, then report only on signals that can
     * be opened. otherwise, report on all signals named in the
     * header file, without attempting to open them.
     */
    public static RecordDescriptor wfdbdesc(String recordName,
					    boolean readable)
    {
	String info, p;
	String infoText;
	int i, msrec = 0, nsig;
	WFDB_SiginfoArray s = null;
	int t;
	RecordDescriptor recordDescriptor;

	int numGroups;


	//
	nsig = wfdb.isigopen(recordName, null, 0);
	if (nsig <= 0)
	{
	    System.out.println("error: unable to open record " + recordName);
	    wfdb.wfdbquit();
	    return null;
	}

	//
	recordDescriptor = new RecordDescriptor();

	// allocate storage for nsig signal information structures
	if (nsig > 0)
	{
	    s = new WFDB_SiginfoArray(nsig);
	}

	// if the 'readable' option is given, report only on signals
	// which can be opened. otherwise, report on all signals named
	// in the header file, without attempting to open them.
	if (nsig > 0 && readable)
	{
	    nsig = wfdb.isigopen(recordName, s.cast(), nsig);
	}
	else if (nsig > 0)
	{
	    nsig = wfdb.isigopen(recordName, s.cast(), -nsig);
	}

	//
	wfdb.setgvmode(wfdb.WFDB_LOWRES);

	//
	t = wfdb.strtim("e");
	if (nsig > 0 && s.getitem(0).getNsamp() != t) // the record is multi-segment
	{
	    msrec = 1;
	    recordDescriptor.multisegmentType =
		(s.getitem(0).getNsamp() == 0) ? "variable-layout" : "fixed-layout";
	}
	else if ((info = wfdb.getinfo(null)) != null)
	{
	    // FIXME: there's an extra newline...
	    recordDescriptor.notes = "";
	    do {
		recordDescriptor.notes += info + "\n";
	    } while ((info = wfdb.getinfo(null)) != null);
	}
	
	//
	if (msrec == 0)
	{
	    recordDescriptor.multisegmentType = "not multi-segment";
	}

	//
	p = wfdb.mstimstr(0);
	if (p.charAt(0) == '[')
	{
	    recordDescriptor.startingTime = p;
	}
	else
	{
	    recordDescriptor.startingTime = "not specified";
	}
	
	if (nsig < 1 || t > 0)
	{
	    recordDescriptor.length = (t > 0) ? wfdb.mstimstr(t).trim() : "0";
	    recordDescriptor.numSampleIntervals = t;
	    recordDescriptor.lengthSpecified = true;
	}
	// FIXME: i am actually unclear about when this clause is used, and how
	//        File.length() is guaranteed to work (e.g. for remote files)
	else if (s.getitem(0).getFmt() != 0)
	{
	    int nbytes;

	    // FIXME: this should be wrapped in a try {}, i think...
	    nbytes = (int)(new File(s.getitem(0).getFname())).length();

	    if (nbytes == 0)
	    {
		recordDescriptor.lengthSpecified = false;
	    }
	    else
	    {
		int framesize = 0;

		nbytes -= wfdb.wfdbgetstart(0);

		for (i = 0; i < nsig && s.getitem(i).getGroup() == 0; ++i)
		{
		    framesize += s.getitem(i).getSpf();
		}
	    
		switch (s.getitem(0).getFmt())
		{
		case 8:
		case 80:
		    t = nbytes / framesize;
		    break;
		    
		default:
		case 16:
		case 61:
		case 160:
		    t = nbytes / (2*framesize);
		    break;

		case 212:
		    t = (2 * nbytes) / (3*framesize);
		    break;
		    
		case 310:
		case 311:
		    t = (3 * nbytes) / (4*framesize);
		    break;
		}
	     
		recordDescriptor.length = wfdb.mstimstr(t);
		recordDescriptor.numSampleIntervals = t;
		recordDescriptor.lengthSpecified = true;
	    }
	}
	else
	{
	    recordDescriptor.lengthSpecified = false;
	}

	//
	recordDescriptor.samplingFrequency = wfdb.sampfreq(null);

	//
	if (nsig < 1)
	{
	    wfdb.wfdbquit();
	    return null;
	}

	//
	if (msrec != 0 && s.getitem(0).getNsamp() != 0)
	{
	    recordDescriptor.segmentLength = wfdb.mstimstr(s.getitem(0).getNsamp()).trim();
	    recordDescriptor.numSegmentSampleIntervals = s.getitem(0).getNsamp();
	}

	recordDescriptor.groups = createGroupDescriptors(s, nsig);
	wfdb.wfdbquit();
	return recordDescriptor;
    }

}

